#! /usr/bin/env python
# -*- coding:Utf-8 -*-

# ================================================================
# Script permettant de (re)démarrer ou d'arrêter le serveur Palyno
# ==== Attention : sous Linux seulement !!! ======================
# Syntaxe : 
#             ./palyserv.py start
#             ./palyserv.py stop
#             ./palyserv.py restart

# Version du 10/01/2008

import os, sys, pwd
from os.path import exists      # test d'existence d'un chemin
from time import sleep

def dataFromConfig(ll, label):
    "renvoie les items qui suivent l'item '[label]\n' dans la liste ll"
    ind = ll.index("[%s]\n" % label) +1
    res =""
    while ind < len(ll):
        li = ll[ind]
        if li[0] =="[" or not li: break
        res +=li
        ind +=1
    return res[:-1]     # Enlever linefeed final

def findOption():
    "renvoie l'option choisie au lancement, ou None"
    options = sys.argv
    if len(options) >1: 
        option = options[1]
        if option in ('start','stop','restart'):
            return option
    print "Usage :  ./palyserv.py [start|stop|restart]"

def findProcessNumber(commande):
    "renvoie le n° de processus (pid) associé à <commande>"
    # (On utilise la commande ps dont on lit le résultat à l'aide d'un
    #  "pipe". La commande ps utilisée renvoie une série de lignes 
    #  correspondant à tous les processus  démarrés par l'utilisateur
    #  "propriétaire". Il faut sélectionner la ligne correspondant à la
    #  commande de lancement de BulréZo. Le 2e champ de cette ligne contient
    #  le numéro de processus recherché).

    # Création d'un <objet-fichier> lié à un "pipe" :
    ofi =os.popen("ps -u %s -f" % (proprio), "r")
    pidList = ofi.readlines()       # lire toutes les lignes => liste
    ofi.close()                     # refermer le pipe
    for pidStr in pidList:          # rechercher la ligne
        if commande in pidStr:      #    contenant la commande
            items = pidStr.split()  # convertir la ligne lue en une liste
            return items[1]         # extraire le pid (2e champ)
    # La commande n'a pas été trouvée dans les lignes examinées :
    return None

def stopper():
    "arrêter le serveur Palyno"
    # Retrouver le n° de processus associé au lancement de Palyno :
    pid = findProcessNumber(programName)
    # "Tuer" ce processus :
    if pid:
        print "Arrêt du serveur %s ..." % (programName), 
        # os.system("kill %s" % (pid))
        # L'instruction ci-après tue tous les processus de l'utilisateur courant
        # dont la commande de lancement se termine par <programName> :
        cmd ='''kill -9 `ps ux | awk '$12=="%s" {print $2}'`''' % programName
        os.system(cmd)
        print " ok."
    else:
        print "%s n'est pas en fonction." % (programName)

def demarrer():
    "démarrer le serveur Palyno"
    print "Démarrage du serveur %s ..." % (programName), 
    sleep(1)
    os.chdir(palynoDir)
    # Trouver un nouveau nom pour le fichier journal de Palyno :
    n = 0
    while 1:
        n += 1
        logFile ="%s_log%s.txt" % (programName, n)
        if not exists(logFile): break
    # démarrer programName, en dirigeant sa sortie vers un nouveau journal :
    os.system("python %s > /dev/null &" % (programName)) 
    print " ok. Répertoire de travail : %s." % (os.getcwd())

#### programme principal : ####

# Récupérer les options fournies dans la ligne de commande :
option = findOption()

########################## personnalisation : ############################
#programName ="palyno.py"		    # nom du script exécutable
programName ="palyno.pyc"		    # nom du script exécutable
# Retrouver le propriétaire déclaré pour ce script :
user_id = os.geteuid()              # propriétaire du processus courant
user_infos = pwd.getpwuid(user_id)  # infos trouvées dans /etc/passwd
proprio = user_infos[0]             # nom de l'utilisateur propriétaire
#palynoDir ="/home/%s" % proprio      # répertoire de travail
#palynoDir ="/home/palyno"            # répertoire de travail
palynoDir =os.getcwd()                # répertoire de travail
##########################################################################

if option =="stop":
    stopper()
elif option =="start":
    demarrer()
elif option =="restart":
    stopper()
    sleep(1)
    demarrer()
sys.exit()
